#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
Windows registry simple API

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.registry"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2023050601"

import re
from windows_tools.registry import *

DATE_REGEX = "[0-9]{4}-[0-9]{2}-[0-9]{2} [0-9]{2}:[0-9]{2}"


def test_get_value():
    product_name = get_value(
        hive=HKEY_LOCAL_MACHINE,
        key=r"SOFTWARE\Microsoft\Windows NT\CurrentVersion",
        value="ProductName",
        arch=KEY_WOW64_32KEY | KEY_WOW64_64KEY,
    )
    print("Product Name: %s" % product_name)
    assert product_name.startswith(
        "Windows"
    ), 'product_name should start with "Windows"'

    key = get_value(
        hive=HKEY_LOCAL_MACHINE,
        key=r"SOFTWARE\Microsoft\Windows NT\CurrentVersion",
        value="ProductName",
        arch=KEY_WOW64_32KEY | KEY_WOW64_64KEY,
        last_modified=True,
    )
    print("Product Name key: %s" % key["value"])
    print("Last modified: %s" % key["last_modified"])
    assert key["value"].startswith(
        "Windows"
    ), 'product_name should start with "Windows"'
    assert re.match(DATE_REGEX, key["last_modified"]), 'Date format invalid"'


def test_get_values():
    uninstall = get_values(
        hive=HKEY_LOCAL_MACHINE,
        key=r"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall",
        names=["DisplayName", "Version"],
        arch=KEY_WOW64_32KEY | KEY_WOW64_64KEY,
    )
    print(uninstall)
    assert len(uninstall) > 0, "get_values() returned an empty object."
    assert isinstance(uninstall[0], dict), "get_values() should return list of dict"
    assert (
        len(uninstall[0]) > 0
    ), "get_values() should return at least one installed product"

    uninstall = get_values(
        hive=HKEY_LOCAL_MACHINE,
        key=r"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall",
        names=["DisplayName", "Version"],
        arch=KEY_WOW64_32KEY | KEY_WOW64_64KEY,
        last_modified=True,
    )
    for uninst in uninstall:
        if uninst[list(uninst.keys())[0]]:
            assert re.match(
                DATE_REGEX, uninst[list(uninst.keys())[0]]["last_modified"]
            ), "Invalid date format"


def test_get_keys():
    uninstall_recursive = get_keys(
        hive=HKEY_LOCAL_MACHINE,
        key=r"SOFTWARE\Microsoft\Windows\CurrentVersion\Uninstall",
        recursion_level=2,
        filter_on_names=["DisplayName", "Version"],
        arch=KEY_WOW64_32KEY | KEY_WOW64_64KEY,
        last_modified=True,
    )
    print(uninstall_recursive)
    assert isinstance(
        uninstall_recursive, dict
    ), "get_keys() should return a dict with entries per key, containing dict or lists"
    assert (
        len(uninstall_recursive) > 0
    ), "get_keys() should return at least one software to that could be uninstalled"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_get_value()
    test_get_values()
    test_get_keys()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of windows_tools module

"""
wmi_queries wrapper and wmi timestamp converter
transforms wmi objects into python dicts
Handles most runtime errors

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "windows_tools.wmi_queries"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__description__ = "Windows WMI query wrapper, wmi timezone converters"
__licence__ = "BSD 3 Clause"
__version__ = "1.0.1"
__build__ = "2024110701"

import logging
import re
from datetime import datetime, timedelta, timezone
import time
from logging.handlers import QueueHandler

# importing queues is only necessary for typing hints
# SimpleQueue does not exist in Python < 3.7
try:
    from queue import SimpleQueue, Queue
except ImportError:
    from queue import Queue
    from queue import Queue as SimpleQueue
from typing import Union

# imports to debug WMI requests with better error messages
import pywintypes
import wmi

logger = logging.getLogger(__intname__)


def wmi_object_2_list_of_dict(
    wmi_objects, depth: int = 1, root: bool = True
) -> Union[dict, list]:
    """
    Return a WMI object as a list of dicts, accepts multiple depth
    Example for Win32_LoggedOnUser().Antecedent.AccountType return is [{'Antecedent': {'AccountType': 512}}]
    Hence
    wmi_handle.Win32_LoggedOnUser()[0].Antecedent.AccountType is equivalent of
    res = wmi_object_2_list_of_dict(wmi_handle.Win32_LoggedOnUser(), 2)
    res[0]['Antecedent']['AccountType']
    """

    result = []

    if root is False:
        dictionary = {}
        try:
            for attribute in wmi_objects.properties:
                try:
                    if depth > 1:
                        dictionary[attribute] = wmi_object_2_list_of_dict(
                            getattr(wmi_objects, attribute), (depth - 1), root=False
                        )
                    else:
                        dictionary[attribute] = getattr(wmi_objects, attribute)
                except TypeError:
                    dictionary[attribute] = None
            return dictionary
        # wmi_object.properties might just be a string depending on the depth. Just return as is in that case
        except AttributeError:
            return wmi_objects

    for wmi_object in wmi_objects:
        dictionary = {}
        for key in wmi_object.properties.keys():
            if depth <= 1:
                try:
                    dictionary[key] = wmi_object.Properties_(key).Value
                except TypeError:
                    dictionary[key] = None
            else:
                # noinspection PyBroadException
                try:
                    dictionary[key] = wmi_object_2_list_of_dict(
                        getattr(wmi_object, key), (depth - 1), root=False
                    )
                # Some keys won't have attributes and trigger pywintypes.com_error and others. Need for bare except
                except Exception:
                    pass
        result.append(dictionary)
    return result


def query_wmi(
    query_str: str,
    namespace: str = "cimv2",
    name: str = "noname",
    depth: int = 1,
    can_be_skipped: bool = False,
    mp_queue: Union[Queue, SimpleQueue] = None,
    debug: bool = False,
    computer: str = "localhost",
) -> Union[list, None]:
    """
    Execute WMI queries that return pre-formatted python dictionaries
    Also allows to pass a queue for logging returns when using multiprocessing
    """
    if mp_queue:
        logging_handler = QueueHandler(mp_queue)
        local_logger = logging.getLogger()
        local_logger.handlers = []
        local_logger.addHandler(logging_handler)
        if debug:
            local_logger.setLevel(logging.DEBUG)
        else:
            local_logger.setLevel(logging.INFO)
    else:
        local_logger = logging.getLogger(__intname__)

    # Multiprocessing also requires to pickle results
    # Since we cannot send wmi objects, we'll convert them to dict first using wmi_object_2_list_of_dict

    # CoInitialize (and CoUninitialize) are needed when using WMI in threads, but not a multiprocessing child
    # pythoncom.CoInitialize()
    local_logger.debug("Running WMI query [%s]." % name)
    # noinspection PyBroadException

    # Full moniker example
    # wmi_handle = wmi.WMI(moniker=r'winmgmts:{impersonationLevel=impersonate,authenticationLevel=pktPrivacy,(LockMemory, !IncreaseQuota)}!\\localhost\root\cimv2/Security/MicrosoftVolumeEncryption')
    try:
        if namespace.startswith("cimv2"):
            wmi_handle = wmi.WMI(
                moniker=r"winmgmts:{impersonationLevel=impersonate,authenticationLevel=pktPrivacy,(LockMemory, !IncreaseQuota)}!\\%s\root\%s"
                % (computer, namespace)
            )
            return wmi_object_2_list_of_dict(wmi_handle.query(query_str), depth)
        elif namespace == "wmi":
            wmi_handle = wmi.WMI(namespace="wmi")
            return wmi_object_2_list_of_dict(wmi_handle.query(query_str), depth)
        elif namespace == "SecurityCenter":
            # Try to fallback to securityCenter v1 for XP
            # noinspection PyBroadException
            try:
                wmi_handle = wmi.WMI(namespace="SecurityCenter2")
            except Exception:
                # noinspection PyBroadException
                try:
                    wmi_handle = wmi.WMI(namespace="SecurityCenter")
                except Exception:
                    logger.info("cannot get securityCenter handle.")
                    return None
            return wmi_object_2_list_of_dict(wmi_handle.query(query_str), depth)
        else:
            local_logger.critical("Bogus query path {}.".format(namespace))
    except pywintypes.com_error:
        if can_be_skipped is not True:
            local_logger.warning(
                "Cannot get WMI query (pywin) {}.".format(name), exc_info=True
            )
            local_logger.debug("Trace:", exc_info=True)
        else:
            local_logger.info("Cannot get WMI query (pywin) {}.".format(name))
    except wmi.x_access_denied:
        if can_be_skipped is not True:
            local_logger.warning(
                "Cannot get WMI request (access) {}.".format(name), exc_info=True
            )
            local_logger.debug("Trace:", exc_info=True)
        else:
            local_logger.info("Cannot get WMI request (access) {}.".format(name))
    except wmi.x_wmi:
        if can_be_skipped is not True:
            local_logger.warning(
                "Cannot get WMI query (x_wmi) {}.".format(name), exc_info=True
            )
            local_logger.debug("Trace:", exc_info=True)
        else:
            local_logger.info("Cannot get WMI query (x_wmi) {}.".format(name))
    except NameError:
        if can_be_skipped is not True:
            local_logger.warning("Cannot get WMI request (name) {}.".format(name))
            local_logger.debug("Trace:", exc_info=True)
        else:
            local_logger.info("Cannot get WMI query (name) {}.".format(name))
    except Exception:
        if can_be_skipped is not True:
            local_logger.warning(
                "Cannot get non skippable WMI request (uncaught) {}.".format(name)
            )
            local_logger.debug("Trace:", exc_info=True)
        else:
            local_logger.info("Cannot get WMI request (uncaught) {}.".format(name))
    return None

    # Only needed when used in threaded environment
    # finally:
    # pythoncom.CoUninitialize()


def get_timezone_offset() -> int:
    is_dst = time.daylight and time.localtime().tm_isdst > 0
    utc_offset = -(time.altzone if is_dst else time.timezone)
    return utc_offset


def get_wmi_timezone_bias() -> str:
    """
    Get current timezone bias in WMI compatible format, eg:
    "UTC+2" = "+120"

    :return: str
    """
    result = query_wmi(
        query_str="SELECT Bias FROM Win32_timezone",
        namespace="cimv2",
        name="windows_tools.wmi_queries.timezonebias",
        depth=1,
        can_be_skipped=False,
    )
    try:
        return str(result[0]["Bias"])
    except (KeyError, IndexError):
        logger.warning("Missing timezone bias info. Using UTC+0.")
        return "0"
    except TypeError:
        logger.warning("Missing timzeone bias. Using UTC+0.")
        return "0"


def utc_datetime_to_cim_timestamp(dt: datetime, localize: bool = True) -> str:
    """
    Creates a WMI compatible timestamp from python datetime object
    """
    if localize:
        timezonebias = get_wmi_timezone_bias()
    else:
        timezonebias = "0"

    if timezonebias[0] == "-":
        tz_bias = timezonebias
    else:
        tz_bias = "+" + timezonebias
    cim_timestamp = dt.strftime("%Y%m%d%H%M%S.%f") + tz_bias
    return cim_timestamp


def cim_timestamp_to_datetime(cim_timestamp: str, utc: bool = True) -> datetime:
    """
    Convert WMI timestamp to python datetime object

    wmi timestamps ALWAYS include timezones, hence are always UTC

    if utc is False, we'll return a datetime object without current timezone
    """
    cim_time, cim_offset = re.split("[+-]", cim_timestamp)
    timestamp = datetime.strptime(cim_time, "%Y%m%d%H%M%S.%f")

    if "+" in cim_timestamp:
        offset = int(cim_offset)
    elif "-" in cim_timestamp:
        offset = -int(cim_offset)
    else:
        offset = 0

    if utc:
        timestamp = timestamp.replace(tzinfo=timezone(timedelta(minutes=offset)))
    else:
        timestamp += timedelta(minutes=offset)
    return timestamp


def create_cim_timestamp_from_now(**kwargs) -> str:
    """
    Creates a WMI compatible timestamp, like "20200818124602.499324+120" from
    current datetime
    Current datetime can be increased / decreased by specifing an offset

    Future timestamps are achieved by specifying positive values, eg days=1
    Past timestamps are achieved by specifying negative values, eg days=-1

    Accepts all kwargs that timedelta accepts
    """
    dt = datetime.utcnow() + timedelta(**kwargs)
    return utc_datetime_to_cim_timestamp(dt)


def create_current_cim_timestamp(hour_offset: int = 0) -> str:  # COMPAT <0.9.5
    """
    Compatibility wrapper for create_cim_timestamp_from_now
    """
    return create_cim_timestamp_from_now(hours=-hour_offset)



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
Find Windows Product key from registry and various

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.product_key"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021031601"

import re

from windows_tools.product_key import *

PRODUCT_KEY_REGEX = r"([A-Z0-9]{5}-){4}[A-Z0-9]{5}"


def test_get_windows_product_key_from_wmi():
    wmi_key = get_windows_product_key_from_wmi()
    print('Key from WMI: "{}"'.format(wmi_key))
    if wmi_key is not None and wmi_key != "":
        assert re.match(
            PRODUCT_KEY_REGEX, wmi_key
        ), "Found product key matches product key format"


def test_get_windows_product_key_from_reg():
    reg_key = get_windows_product_key_from_reg()
    print('Key from REG: "{}"'.format(reg_key))
    if reg_key is not None and reg_key != "":
        assert re.match(
            PRODUCT_KEY_REGEX, reg_key
        ), "Found product key matches product key format"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_get_windows_product_key_from_wmi()
    test_get_windows_product_key_from_reg()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
Privilege enabling / disabling

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.securityprivilege"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021021601"

from windows_tools.securityprivilege import *


def test_enable_privilege():
    print("Enabling SeSecurityPrivilege")
    enable_privilege("SeSecurityPrivilege")


def test_disable_privilege():
    print("Disabling SeSecurityPrivilege")
    disable_privilege("SeSecurityPrivilege")


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_enable_privilege()
    test_disable_privilege()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
windows server identification

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.server"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021021601"

from windows_tools.server import *


def test_is_windows_server():
    """
    This test is very simple since we won't know for sure if actually running on server
    So we only test functionality here, actual tests have been done manually
    """
    is_server = is_windows_server()
    print("This machine is a windows server: ", is_server)
    assert is_server is True or is_server is False, "is_server should be a boolean"


def test_is_rds_server():
    """
    This test is very simple since we won't know for sure if actually running on rds server
    So we only test functionality here, actual tests have been done manually
    """
    is_rds = is_rds_server()
    print("This machine is a windows RDS server: ", is_rds)
    assert is_rds is True or is_rds is False, "is_rds should be a boolean"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_is_windows_server()
    test_is_rds_server()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
Windows NTFS & ReFS ownership/ACL tools

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.file_utils"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021021501"

import os
import shutil
from random import random

from windows_tools.file_utils import *
from windows_tools.users import *

TEST_DIR = os.path.join(
    os.environ.get("TEMP", r"C:\Windows\Temp"),
    __file__ + "TestDir{}".format(int(random() * 100000)),
)
TEST_FILE = os.path.join(
    os.environ.get("TEMP", r"C:\Windows\Temp"),
    os.path.basename(__file__) + ".Test{}".format(int(random() * 100000)),
)


def test_get_ownership():
    """
    Again, we can only assume that all tests are done on 64 bit systems nowadays
    """
    local_windows_dir = r"C:\Windows"

    owner = get_ownership(local_windows_dir)
    print("{} directory owner; {}.".format(local_windows_dir, owner))
    assert (
        owner[0] == "TrustedInstaller"
    ), "{}} file owner bogus, should be TrustedInstaller since win7".format(
        local_windows_dir
    )
    assert (
        owner[1] == "NT SERVICE"
    ), "{} file owner should be of group NT SERVICE since Win7".format(
        local_windows_dir
    )
    assert owner[2] == 5, "{} file owner rights should be 5 since Win7".format(
        local_windows_dir
    )


def test_take_ownership():
    """
    Let's create a file, check it's owner, than change it and test again
    """
    if os.path.isfile(TEST_FILE):
        os.remove(TEST_FILE)
    with open(TEST_FILE, "w") as file_handle:
        file_handle.write("SOME TEXT")

    owner = get_ownership(TEST_FILE)
    print("Current file owner: ", TEST_FILE, owner)

    # In order to use set_file_owner, we need a PySID object as owner
    system_account = get_pysid(well_known_sids(username="System"))
    result = take_ownership(TEST_FILE, owner=system_account, force=True)

    new_owner = get_ownership(TEST_FILE)
    print("Current file owner: ", TEST_FILE, new_owner)
    new_owner_sid = get_pysid_from_username(new_owner[0])
    assert result is True, 'take_ownership result is not true: "{}"'.format(result)
    assert (
        new_owner_sid[0].__str__() == "PySID:S-1-5-18"
    ), "File should now be owned by system account"
    assert (
        new_owner != owner
    ), "Earlier owner and current owner should differ unless tests are run as system"

    if os.path.isfile(TEST_FILE):
        os.remove(TEST_FILE)


def test_take_ownership_recursive():
    if os.path.isdir(TEST_DIR):
        shutil.rmtree(TEST_DIR)
    os.makedirs(TEST_DIR)

    print("Test take_ownership_recursive()")
    owner = get_pysid(well_known_sids(username="Everyone"))
    take_ownership_recursive(TEST_DIR, owner=owner)

    shutil.rmtree(TEST_DIR)


def test_easy_permissions():
    easy_perm = easy_permissions("R")
    assert easy_perm == -2147483648, "Permission bitmask wrong for R"
    easy_perm = easy_permissions("RX")
    assert easy_perm == -1610612736, "Permission bitmask wrong for RX"
    easy_perm = easy_permissions("RWX")
    assert easy_perm == -536870912, "Permission bitmask wrong for RWX"
    easy_perm = easy_permissions("M")
    assert easy_perm == -536870912, "Permission bitmask wrong for M"
    easy_perm = easy_permissions("F")
    assert easy_perm == 268435456, "Permission bitmask wrong for F"
    try:
        easy_perm = easy_permissions("")
    except ValueError:
        assert easy_perm != 307302933, "Bogus permission should have happened"


def test_set_acls():
    """
    Really basic test, TODO: improve with inheritance tests
    """
    if os.path.isfile(TEST_FILE):
        os.remove(TEST_FILE)
    with open(TEST_FILE, "w") as file_handle:
        file_handle.write("SOME TEXT")

    print("Test setting ACLs")
    try:
        set_acls(
            TEST_FILE,
            user_list=[whoami()],
            owner=whoami(),
            permissions=easy_permissions("F"),
        )
    except Exception as e:
        assert False, "set_acls should not create an exception: {}".format(e)

    os.remove(TEST_FILE)


def test_get_paths_recursive_and_fix_permissions():
    """
    Really basic tests too
    """
    print("Test get_files_recursive_and_fix_permissions")

    if os.path.isdir(TEST_DIR):
        get_paths_recursive_and_fix_permissions(
            TEST_DIR, owner=whoami(), permissions=easy_permissions("RWX")
        )
        shutil.rmtree(TEST_DIR)
    os.makedirs(TEST_DIR)
    with open(
        os.path.join(TEST_DIR, os.path.basename(__file__) + ".tst"), "w"
    ) as file_handle:
        file_handle.write("SOMETEXT")

    try:
        set_acls(
            TEST_DIR,
            user_list=[whoami()],
            owner=whoami(),
            permissions=easy_permissions("R"),
            inherit=False,
            inheritance=False,
        )
    except Exception as e:
        assert False, "set_acls had an exception: {}".format(e)

    # Normally, we should not be able to delete the fil since we set permissions as RX only
    try:
        # shutil.rmtree(TEST_DIR)
        pass
    except PermissionError as e:
        print("We have a permission error: ".format(e))
    else:
        pass
        # assert False, 'We should have a permission error here'

    try:
        get_paths_recursive_and_fix_permissions(
            TEST_DIR, permissions=easy_permissions("F")
        )
    except Exception as e:
        assert (
            False
        ), "get_file_recursive_and_fix_permissions had an exception: {}".format(e)

    try:
        shutil.rmtree(TEST_DIR)
    except PermissionError:
        assert False, "This time, we should not have a permission error here"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_get_ownership()
    test_take_ownership()
    test_take_ownership_recursive()
    test_easy_permissions()
    test_set_acls()
    # test_get_files_recursive_and_fix_permissions()  # TODO: fix set_acls(inherit=False) to not copy DACLs in order
    # to write this test correctly



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
Simple registry check for windows firewall status

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.windows_firewall"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021021601"

from windows_tools.windows_firewall import *


def test_is_firewall_active():
    """
    Too simple check here, has been tested manually
    We won't know for sure in advance if firewall should be enabled
    """
    result = is_firewall_active()
    print("Windows firewall status: ", result)
    assert result is True or result is False, "Result should be a boolean"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_is_firewall_active()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
Retrieve bitlocker status and protector keys

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.bitlocker"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021021501"

import logging
from windows_tools.bitlocker import *


def test_get_bitlocker_full_status():
    """
    There's no real test I can write for bitlocker unless the target machine has a bitlocker enabled drive
    So here we are writing basic tests
    """
    result = get_bitlocker_full_status()
    print(result)
    assert (
        result["C:"]["status"] is not False
    ), "C: bitlocker status must be None or a string"
    assert (
        result["C:"]["protectors"] is not False
    ), "C: protectors should be None or a string"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    logger.addHandler(logging.StreamHandler())
    logger.setLevel(logging.DEBUG)
    test_get_bitlocker_full_status()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
Using windows signtool.exe to add authenticodes to executables

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.signtool"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2024110701"

import os
import shutil
import random
import string


from windows_tools.powershell import PowerShellRunner
from windows_tools.signtool import *


def running_on_github_actions():
    """
    This is set in github actions workflow with
          env:
        RUNNING_ON_GITHUB_ACTIONS: true
    """
    return os.environ.get("RUNNING_ON_GITHUB_ACTIONS") == "true"  # bash 'true'


def pw_gen(size: int = 16, chars: list = string.ascii_letters + string.digits) -> str:
    return "".join(random.choice(chars) for _ in range(size))


def create_test_certificate():
    """
    Create a code signing certificate in order to make real signing tests
    Needs Windows 8.1+ (powershell 4.0) in order to work
    Older Win7 systems will fail this test because they rely on makecert.exe
    """

    # Github actions doesn't have a CERT:\ shortcut, so we'll fail with that test
    if running_on_github_actions():
        return None

    cert_location = os.path.join(
        os.environ.get("TEMP", r"C:\Windows\Temp"), "self_signed_test_authenticode.pfx"
    )
    dns_name = "acme.corp"
    password = pw_gen()

    create_command = r"$cert = New-SelfSignedCertificate -DnsName {} -Type CodeSigning -CertStoreLocation Cert:\CurrentUser\My".format(
        dns_name
    )
    pw_command = '$CertPassword = ConvertTo-SecureString -String "{}" -Force –AsPlainText'.format(
        password
    )
    export_command = r'Export-PfxCertificate -Cert "cert:\CurrentUser\My\$($cert.Thumbprint)" -FilePath "{}" -Password $CertPassword'.format(
        cert_location
    )
    full_command = "%s; if ($?) {%s}; if ($?) {%s}" % (
        create_command,
        pw_command,
        export_command,
    )

    PS = PowerShellRunner()
    exit_code, output = PS.run_command(full_command)
    if exit_code == 0:
        print(
            'Created test cert at {} with password "{}"'.format(cert_location, password)
        )
        return cert_location, password
    print("ERROR:", output)
    return None


def test_signtool_path():

    if running_on_github_actions():
        return None
    signer = SignTool(
        certificate=None,
        pkcs12_password="None",
        authority_timestamp_url="http://timestamp.digicert.com",
    )
    print("Path for x86 signtool", signer.detect_signtool("x86"))
    print("Path for x86 signtool", signer.detect_signtool("x64"))
    # We do not assert here since we won't have an SDK on the test machine


def test_signer():
    """
    Some signtool compatible timestamp URLs (as of 2021051401)

    http://timestamp.verisign.com/scripts/timstamp.dll
    http://timestamp.globalsign.com/scripts/timstamp.dll
    http://timestamp.comodoca.com/authenticode
    http://timestamp.digicert.com
    """

    if running_on_github_actions():
        return None
    cert, password = create_test_certificate()
    signer = SignTool(
        certificate=cert,
        pkcs12_password=password,
        authority_timestamp_url="http://timestamp.digicert.com",
    )

    # Let's copy then sign some executable, let's say cmd.exe
    # Using copyfile because we don't want metadata, permissions, buffer nor anything else
    source = os.path.join(
        os.environ.get("SYSTEMROOT", r"C:\Windows"), "system32", "cmd.exe"
    )
    destination = os.path.join(os.environ.get("TEMP", r"C:\Windows\Temp"), "cmd.exe")
    shutil.copyfile(source, destination)

    print("Now signing {}".format(destination))

    try:
        signer.sign(destination, 32)
    except AttributeError as exc:
        # AttributeError because we provided an empty authority timestamp url
        print("(NORMAL BEHAVIOR IN TESTS) Signing failed with: %s" % exc)
        assert (
            False
        ), "Failed to signe test executable with our superb self signed certificate"
    os.remove(destination)
    os.remove(cert)


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_signtool_path()
    test_signer()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
Microsoft Powershell runner class

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.powershell"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021021601"

from windows_tools.powershell import *


def test_PowerShellRunner():
    """ """

    P = PowerShellRunner()
    interpreter = P.powershell_interpreter
    print("Powershell interpreter: ", interpreter)
    assert interpreter is not None, "Powershell interpreter not found"
    version = P.get_version()
    print("Powershell version: ", version)
    assert isinstance(
        version, tuple
    ), "Powerhsell version should be a tuple of two ints"
    assert 0 < int(version[0]) < 10, "Powershell "

    exit_code, output = P.run_command("Get-ComputerInfo | Select OsInstallDate")
    assert exit_code == 0, "Command exit code should be zero"
    print(output)


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_PowerShellRunner()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
Simple check if Windows is 64 bit

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.bitness"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021050601"

from windows_tools.bitness import *


def test_is_64bit():
    """
    Again, we can only assume that all tests are done on 64 bit systems nowadays
    """
    assert is_64bit() is True, "tests should probably be run on a 64 bit system"
    print("Is this a bit system ?", is_64bit())


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_is_64bit()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
Windows NTFS & ReFS ownership/ACL tools

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.logical_disks"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021060201"

import subprocess

from windows_tools.logical_disks import (
    _get_logical_disks_plaintest,
    _get_logical_disks_win32api,
    _get_logical_disks_psutil,
    get_logical_disks,
)


def test__get_logical_disks_plaintest():
    result = _get_logical_disks_plaintest()
    print("Found disks via plaintest: ", result)
    assert isinstance(result, list), "get_logical_disks() should return list of str"
    assert "C:" in result, "We should at least have systemdrive C:"


def test__get_logical_disks_win32api():
    result = _get_logical_disks_win32api()
    print("Found disks via win32api: ", result)
    assert isinstance(result, list), "get_logical_disks() should return list of str"
    assert "C:" in result, "We should at least have systemdrive C:"


def test__get_logical_disks_psutil():
    result = _get_logical_disks_psutil()
    print("Found disks via psutil: ", result)
    assert isinstance(result, list), "get_logical_disks() should return list of str"
    assert "C:" in result, "We should at least have systemdrive C:"


def test_get_logical_disks():
    # Create a network
    command = r"net use Q: /DELETE"
    try:
        subprocess.check_output(command, shell=True, timeout=4)
    except subprocess.CalledProcessError:
        pass
    command = r"net use Q: \\localhost\c$\windows\temp"
    print(command)
    output = subprocess.check_output(command, shell=True, timeout=4)
    print("Creating network drive: ", output)
    no_net = get_logical_disks(include_network_drives=False)
    net_result = get_logical_disks(include_network_drives=True)
    # Delete the network drive
    command = "net use Q: /DELETE"
    output = subprocess.check_output(command, shell=True, timeout=4)
    print("Deleting network drive: ", output)
    print("Found disks from meta function without network drives: ", no_net)
    print("Found disks from meta function including network drives: ", net_result)

    assert len(no_net) < len(
        net_result
    ), "List without network drives should be smaller than list with network drives"
    assert isinstance(no_net, list), "get_logical_disks() should return list of str"
    assert "C:" in no_net, "We should at least have systemdrive C:"
    assert isinstance(net_result, list), "get_logical_disks() should return list of str"
    assert "C:" in net_result, "We should at least have systemdrive C:"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test__get_logical_disks_plaintest()
    test__get_logical_disks_win32api()
    test__get_logical_disks_psutil()
    test_get_logical_disks()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
virtualization identification

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.virtualization"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021021601"

from windows_tools.virtualization import *


def test_get_relevant_platform_info():
    platform_info = get_relevant_platform_info()
    print(platform_info)
    assert isinstance(platform_info, dict), "platform info should ba a dictionnary"
    assert isinstance(
        next(iter(platform_info.values())), dict
    ), "Virtualization check is not a sub-dictionnary"
    assert isinstance(
        platform_info["computersystem"]["Manufacturer"], str
    ), "computer manufacturer should be found"
    assert isinstance(
        platform_info["baseboard"]["Manufacturer"], str
    ), "baseboard manufacturer should be found"
    assert isinstance(
        platform_info["bios"]["Manufacturer"], str
    ), "bios manufacturer should be found"
    assert isinstance(
        platform_info["diskdrive"]["DeviceID"], str
    ), "computer should at least have one physical disk"


def test_check_for_virtualization():
    """
    Simple checks since we won't know for sure what platform we run on
    """
    is_virtual, hypervisor = check_for_virtualization(get_relevant_platform_info())
    print("Virt check: ", is_virtual, hypervisor)
    assert is_virtual is True or is_virtual is False, "virt check should be a boolean"
    assert isinstance(hypervisor, str), "virt variant should be a string"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_get_relevant_platform_info()
    test_check_for_virtualization()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
antivirus functions help to obtain Windows security antivirus state

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.antivirus"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021020901"

from windows_tools.antivirus import (
    securitycenter_get_product_type,
    securitycenter_get_product_update_state,
    securitycenter_get_product_exec_state,
    get_installed_antivirus_software,
)


# Those states are obtained via WMI securitycenter2/antivirusproduct & securitycenter2/firewallproduct
AV_DISABLED_AND_UP_TO_DATE = 262144
AV_ENABLED_AND_UP_TO_DATE = 266240
FW_ENABLED = 69632
FW_DISABLED = 65536
AV_BOGUS_STATE = 307302933
AV_DEFENDER_DISABLED_AND_UP_TO_DATE = 393472
AV_DEFENDER_ENABLED_AND_OUT_OF_DATE = 397584
AV_DEFENDER_ENABLED_AND_UP_TO_DATE = 397568


def test_securitycenter_get_product_type():
    assert (
        securitycenter_get_product_type(AV_DISABLED_AND_UP_TO_DATE) == "Antivirus"
    ), "Product type should be antivirus"
    assert (
        securitycenter_get_product_type(AV_ENABLED_AND_UP_TO_DATE) == "Antivirus"
    ), "Product type should be antivirus"
    assert (
        securitycenter_get_product_type(FW_ENABLED) == "Firewall"
    ), "Product type should be firewall"
    assert (
        securitycenter_get_product_type(FW_DISABLED) == "Firewall"
    ), "Product type should be firewall"
    try:
        securitycenter_get_product_type(AV_BOGUS_STATE)
    except ValueError:
        pass
    else:
        assert False, "Bogus AV state should trigger an error"


def test_securitycenter_get_product_exec_state():
    assert (
        securitycenter_get_product_exec_state(AV_DISABLED_AND_UP_TO_DATE) is False
    ), "Product state should be disabled"
    assert (
        securitycenter_get_product_exec_state(AV_ENABLED_AND_UP_TO_DATE) is True
    ), "Product state should be enabled"
    assert (
        securitycenter_get_product_exec_state(FW_ENABLED) is True
    ), "Product state should be enabled"
    assert (
        securitycenter_get_product_exec_state(FW_DISABLED) is False
    ), "Product state should be disabled"


def test_securitycenter_get_product_update_state():
    assert (
        securitycenter_get_product_update_state(AV_DISABLED_AND_UP_TO_DATE) is True
    ), "Product state should be uptodate"
    assert (
        securitycenter_get_product_update_state(AV_ENABLED_AND_UP_TO_DATE) is True
    ), "Product state should be uptodate"


def test_get_installed_antivirus_software():
    result = get_installed_antivirus_software()
    print("Antivirus software:\n")
    print(result)
    assert isinstance(result, list) is True, "AV product list should be a list"
    if result != []:
        assert isinstance(result[0]["name"], str) is True, (
            "First found AV product name should be a string."
            "Please execute this test on a Windows machine with an AV engine installed,"
            " otherwise disable this test. PS: Windows defender does not count as AV engine"
        )


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_securitycenter_get_product_type()
    test_securitycenter_get_product_exec_state()
    test_securitycenter_get_product_update_state()
    test_get_installed_antivirus_software()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
users handles SID to username scenarios

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.users"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021101101"

from windows_tools.users import *


def test_well_known_sids():
    admins_group_sid = well_known_sids(username="Administrators")
    assert admins_group_sid == "S-1-5-32-544", (
        'well_known_sids(username="Administrators") ' 'should return SID "S-1-5-32-544"'
    )

    admins_group_name = well_known_sids(sid="S-1-5-32-544")
    assert (
        admins_group_name == "Administrators"
    ), 'well_known_sids(sid="S-1-5-32-544") should return "Administrators"'


def test_whoami():
    current_user = whoami()
    assert isinstance(current_user, str), "Current username should be a string"


def test_get_username_from_sid():
    builtin_admin = get_username_from_sid("S-1-5-32-544")
    print("Builtin Admins: ", builtin_admin)
    assert builtin_admin[0][0:5] == "Admin", (
        r'BUILINT\Administrators group name should begin with "Admin".'
        "Careful, this is not always valid depending on the system locale"
    )
    assert (
        builtin_admin[1] == "BUILTIN"
    ), r"BUILTIN\Administrators domain should be BUILTIN"
    assert builtin_admin[2] == 4, r"BUILTIN\Administrators type shoud be 4"

    system_account = get_username_from_sid("S-1-5-18")
    print("System account: ", system_account)
    assert system_account[0][0:3].upper() == "SYS", (
        "System account name should begin with Sys."
        " Careful, this is not always valid dpeending on the system locale"
    )
    # Again, we make the assumption that all locale system account domains will contain 'NT' and 'AUT'
    # but not 'AUTH' (eg french 'AUTORITE NT')
    assert "AUT" in system_account[1] and "NT" in system_account[1], (
        "System account domain should be NT "
        "AUTHORITY or something alike depending "
        "on system locale"
    )
    assert system_account[2] == 5, "System account type should be 5"


def test_get_pysid():
    current_user = get_pysid()
    # We cannot check if current_user is a PySID class object since we don't have the definition of that class
    # Let's fallback to some basic test anyway
    assert (
        current_user.__class__.__name__ == "PySID"
    ), "Returned object should be a PySID"
    assert (
        "PySID:S-1-5-21-" in current_user.__str__()
    ), "Current user does not seem to be a valid PySID object"
    print("Current User SID: {}".format(current_user))
    system_account = get_pysid("S-1-5-18")
    print("System Account SID: {}".format(system_account))
    assert (
        system_account.__class__.__name__ == "PySID"
    ), "Returned object should be a PySID"
    assert (
        "PySID:S-1-5-18" == system_account.__str__()
    ), "System account PySID does not seem valid"

    admins_group_by_sid = get_pysid("S-1-5-32-544")
    admins_group_name, _, _ = get_username_from_sid("S-1-5-32-544")
    admins_group_by_name = get_pysid(admins_group_name)
    assert (
        admins_group_by_sid.__class__.__name__ == "PySID"
    ), "Returned object should be a PySID"
    assert (
        admins_group_by_name == admins_group_by_sid
    ), "Both admin group lookups should be equal"


def test_get_pysid_from_username():
    admins_group_name, _, _ = get_username_from_sid("S-1-5-32-544")
    admins_group_sid = get_pysid_from_username(admins_group_name)
    print("Admin group: ", admins_group_sid)
    assert (
        admins_group_sid[0].__class__.__name__ == "PySID"
    ), "Returned object should be a PySID"
    assert (
        "PySID:S-1-5-32-544" in admins_group_sid[0].__str__()
    ), "get_binary_user_info should return a PySID as first element of the returned tuple"


def test_get_local_group_members():
    local_group_members = get_local_group_members(group_sid="S-1-5-32-545")
    print(local_group_members)
    assert local_group_members[0]["name"] != "", "No name in local group membership"


def test_is_user_local_admin():
    is_admin = is_user_local_admin()
    print("Is Admin", is_admin)
    assert isinstance(is_admin, bool), "Result is not a bool"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_well_known_sids()
    test_whoami()
    test_get_username_from_sid()
    test_get_pysid()
    test_get_pysid_from_username()
    test_get_local_group_members()
    test_is_user_local_admin()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
Tool to run as another user

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.impersonate"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021021601"

import subprocess
from random import random

import win32api

from windows_tools.impersonate import *


def test_ImpersonateWin32Sec():
    # create a test user
    TEST_USERNAME = "test_user_{}".format(int(random() * 100000))
    TEST_PASSWORD = "test_pw_{}".format(int(random() * 100000))

    command = "net user {} {} /ADD".format(TEST_USERNAME, TEST_PASSWORD)
    print(command)
    output = subprocess.check_output(command, shell=True, timeout=4)
    print("Creating test user: ", output)

    current_user = win32api.GetUserName()
    print("Current  user: ", current_user)
    with ImpersonateWin32Sec(
        domain=".", username=TEST_USERNAME, password=TEST_PASSWORD
    ):
        imp_current_user = win32api.GetUserName()
        print("Current impersonated user: ", current_user)
        assert (
            current_user != imp_current_user
        ), "Current user and test user should be different"
        assert (
            imp_current_user == TEST_USERNAME
        ), "Impersonated user should be test user"

    command = "net user {} /DELETE".format(TEST_USERNAME)
    output = subprocess.check_output(command, shell=True, timeout=4)
    print("Removing test user: ", output)
    current_user = win32api.GetUserName()
    print("Current  user: ", current_user)

    assert (
        current_user != imp_current_user
    ), "Current user and test user should be different now"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_ImpersonateWin32Sec()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
Get updates installed by Windows Update, including those the QFE doesn't list

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.updates"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2021 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021100502"


import os
from windows_tools.updates import *


def running_on_github_actions():
    """
    This is set in github actions workflow with
          env:
        RUNNING_ON_GITHUB_ACTIONS: true
    """
    return os.environ.get("RUNNING_ON_GITHUB_ACTIONS") == "true"  # bash 'true'


def test_get_windows_updates_reg():
    updates = get_windows_updates_reg()
    assert isinstance(updates, list), "Result should be a list"


def test_get_windows_updates_wmi():
    updates = get_windows_updates_wmi()
    assert isinstance(updates, list), "Result should be a list"


def test_get_windows_updates_com():
    # Does not work on github actions
    # pywintypes.com_error: (-2147352567, 'Exception occurred.', (0, None, None, None, 0, -2147023838), None)
    if running_on_github_actions():
        return None
    updates = get_windows_updates_com()
    assert isinstance(updates, list), "Result should be a list"


def test_get_all_windows_updates():
    # Does not work on github actions
    # pywintypes.com_error: (-2147352567, 'Exception occurred.', (0, None, None, None, 0, -2147023838), None)
    if running_on_github_actions():
        return None
    updates = get_windows_updates()

    assert isinstance(updates, list), "Result should be a list"

    for update in updates:
        print(update)


def test_get_windows_updates_filtered():
    # Does not work on github actions
    # pywintypes.com_error: (-2147352567, 'Exception occurred.', (0, None, None, None, 0, -2147023838), None)
    if running_on_github_actions():
        return None
    updates = get_windows_updates(filter_duplicates=True)

    assert isinstance(updates, list), "Result should be a list"

    already_seen_titles = []
    already_seen_kb = []
    for update in updates:
        if update["title"] not in already_seen_titles:
            already_seen_titles.append(update["title"])
        elif update["title"] and update["title"] in already_seen_titles:
            print(update)
            assert False, "We have a title double"
        if update["kb"] not in already_seen_kb:
            already_seen_kb.append(update["kb"])
        elif update["kb"] and update["kb"] in already_seen_kb:
            print(update)
            assert False, "We have a double KB"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_get_windows_updates_reg()
    test_get_windows_updates_wmi()
    test_get_windows_updates_com()
    test_get_all_windows_updates()
    test_get_windows_updates_filtered()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
Microsoft Office identification

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.office"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021021601"

from windows_tools.office import *


def test_get_office_version():
    """
    This is a bogus test that probably will not work on test machines without office installed
    Hence we allow None as return value too, rendering the test not very useful
    """

    office_version, click_and_run = get_office_version()
    print("Office version detected: {} {}".format(office_version, click_and_run))
    assert office_version in list(KNOWN_VERSIONS.values()) + [
        "2016",
        "2019",
        "O365",
        None,
    ], "Bogus office version detected"


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_get_office_version()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
wmi_queries wrapper
transforms wmi objects into python dicts
Handles most runtime errors


Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.wmi_queries"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2023050601"


from windows_tools.wmi_queries import *

CIM_TIMESTAMP_REGEX = (
    r"[0-9]{4}[0-1][0-9][0-3][0-9][0-1][0-9]([0-5][0-9]){2}\.[0-9]{5,6}(\+|-)[0-9]{1,3}"
)


def test_wmi_object_2_list_of_dict():
    print("Testing WMI object conversion")
    raw_result = wmi.WMI(namespace="cimv2").Win32_LoggedOnUser()
    assert isinstance(
        raw_result[0], wmi._wmi_object
    ), "Raw result should be a WMI object"
    result = wmi_object_2_list_of_dict(raw_result)
    assert isinstance(result[0], dict), "Converted result should be a dict"
    try:
        assert (
            result[0]["Antecedent"]["AccountType"] is not None
        ), "Should not exist here"
    except TypeError:
        pass
    else:
        assert False, "First conversion should not have a depth more than 1"
    two_depth_result = wmi_object_2_list_of_dict(raw_result, depth=2)
    # SID should exist regardless of the user account we're making the wmi request with
    assert (
        two_depth_result[0]["Antecedent"]["SID"] is not None
    ), "Two depth result should have AccountType"


def test_query_wmi():
    """
    May fail on elder OS without bitlocker Win32_EncyptableVolume class
    """
    print("Testing WMI query Operating system")
    result = query_wmi(
        "SELECT * FROM Win32_OperatingSystem",
        "cimv2",
        "test_query",
        can_be_skipped=False,
    )
    assert (
        result[0]["Status"] == "OK"
    ), "WMI Query for Win32_OperatingSystem: status != OK"

    print("Testing WMI query EncryptableVolume")
    result = query_wmi(
        "SELECT * FROM Win32_EncryptableVolume",
        "cimv2/Security/MicrosoftVolumeEncryption",
        "test",
    )
    assert isinstance(result[0]["ConversionStatus"], int), (
        "Bitlocker query should give a conversionStatus, " "do we run as admin ?"
    )


def test_get_wmi_timezone_bias():
    """
    bias is what Microsoft calls the minute difference (signed) from UTC time
    """
    bias = int(get_wmi_timezone_bias())
    print("Current timzeone bias: ", bias)
    assert (
        -(23 * 60) < bias < (23 * 60)
    ), "Timezone bias should be in -23 hours up to +23 hours"


def test_cim_timestamp_to_datetime():
    print("Testing cim timestamp to datetime object")

    cim_ts = "20201103225935.123456+0"
    dt = cim_timestamp_to_datetime(cim_ts)
    print("Converted timestamp: ", dt.timestamp())
    assert isinstance(dt, datetime) is True, "Timestamp null TZ conversion failed"
    assert (
        dt.timestamp() == 1604444375.123456
    ), "cim timestamp to timestamp conversion failed"

    cim_ts = "20201103225935.123456+240"
    dt = cim_timestamp_to_datetime(cim_ts)
    print("Converted timestamp: ", dt.timestamp())
    assert (
        isinstance(dt, datetime) is True
    ), "Timestamp with negative TZ conversion failed"
    assert (
        dt.timestamp() == 1604429975.123456
    ), "cim timestamp to timestamp conversion failed"


def test_utc_datetime_to_cim_timestamp():
    print("Test datetime to cim timestamp")

    dt = datetime(2021, 2, 17, 11, 35, 31, 228381)
    cim_timestamp = utc_datetime_to_cim_timestamp(dt)
    print("Created CIM timestamp: ", cim_timestamp)
    assert re.match(CIM_TIMESTAMP_REGEX, cim_timestamp), "Bogus cim timestamp"
    # We don't check for timezone bias here since this test would be different from machine to machine
    assert "20210217113531.228381" in cim_timestamp, "Cim timestamp has wrong date"


def test_create_cim_timestamp_from_now():
    print("Test create current cim timestamp")

    cim_ts = create_cim_timestamp_from_now(hours=0)
    print("Created CIM timestamp: ", cim_ts)
    curr_dt = datetime.utcnow()
    dt = cim_timestamp_to_datetime(cim_ts)
    print("Converted CIM timestamp: ", dt.timestamp())
    print("Current system UTC timestamp: ", curr_dt.timestamp())
    assert isinstance(dt, datetime) is True, "cim timestamp creation failed failed"
    assert dt.year == curr_dt.year, "cim timestamp creation failed failed"
    assert dt.month == curr_dt.month, "cim timestamp creation failed failed"
    assert dt.day == curr_dt.day, "cim timestamp creation failed failed"
    assert dt.hour == curr_dt.hour, "cim timestamp creation failed failed"
    # Too precise, will fail often
    # Can still fail when cim_ts is computed at the last second of an hour
    # assert dt.minute == curr_dt.minute, 'cim timestamp creation failed failed'
    # assert dt.second == curr_dt.second, 'cim timestamp creation failed failed'

    cim_ts = create_cim_timestamp_from_now(hours=-3)
    curr_dt = datetime.utcnow()
    dt = cim_timestamp_to_datetime(cim_ts)
    assert isinstance(dt, datetime) is True, "cim timestamp creation failed failed"
    assert dt.year == curr_dt.year, "cim timestamp creation failed failed"
    assert dt.month == curr_dt.month, "cim timestamp creation failed failed"
    assert dt.day == curr_dt.day, "cim timestamp creation failed failed"
    assert dt.hour + 3 == curr_dt.hour, "cim timestamp creation failed failed"
    # Too precise, will fail often
    # Can still fail when cim_ts is computed at the last second of an hour
    # assert dt.minute == curr_dt.minute, 'cim timestamp creation failed failed'
    # assert dt.second == curr_dt.second, 'cim timestamp creation failed failed'


if __name__ == "__main__":
    print("Example code for %s, %s" % (__intname__, __build__))
    test_wmi_object_2_list_of_dict()
    test_query_wmi()
    test_get_wmi_timezone_bias()
    test_cim_timestamp_to_datetime()
    test_utc_datetime_to_cim_timestamp()
    test_create_cim_timestamp_from_now()



#! /usr/bin/env python
#  -*- coding: utf-8 -*-
#
# This file is part of command_runner module

"""
List of installed software from Uninstall registry keys, 32 and 64 bits

Versioning semantics:
    Major version: backward compatibility breaking changes
    Minor version: New functionality
    Patch version: Backwards compatible bug fixes

"""

__intname__ = "tests.windows_tools.installed_software"
__author__ = "Orsiris de Jong"
__copyright__ = "Copyright (C) 2020-2024 Orsiris de Jong"
__licence__ = "BSD 3 Clause"
__build__ = "2021021601"

import subprocess
from random import random

from windows_tools.installed_software import *


def test_get_installed_software():
    # Add temporary software entries
    fake_software_name = "MyTestSoftwareregEntry{}".format(int(random() * 100000))
    command = (
        r"reg add HKLM\Software\Microsoft\Windows\CurrentVersion\Uninstall\Test_windows_tools.installed_software"
        r" /v DisplayName /t REG_SZ /d {} /f".format(fake_software_name)
    )
    print(command)
    output = subprocess.check_output(command, shell=True, timeout=4)
    print("Creating fake software entry: ", output)
    print("Listing installed software")
    result = get_installed_software()
    assert isinstance(result, list)
    assert isinstance(
        result[0], dict
    ), "get_installed_software() should return list of dict"
    fake_software_present = False
    for entry in result:
        print(entry)
        if entry["name"] == fake_software_name:
            fake_software_present = True

    command = (
        r"reg delete"
        r" HKLM\Software\Microsoft\Windows\CurrentVersion\Uninstall\Test_windows_tools.installed_software /f"
    )
    output = subprocess.check_output(command, shell=True, timeout=4)
    print("Deleting fake software entry: ", output)
    print("Fake software entry present: ", fake_software_present)
    assert fake_software_present, "Fake software entry is not present"


if __name__ == "__main__":
    test_get_installed_software()
